import os

class Paths:
    def __init__(self, hdl_str: str, module_dir_path: str) -> None: ...
    def get_meta(self, key: str) -> os.PathLike: ...
    def get_template(self, key: str) -> os.PathLike: ...
    def get_gen(self, key: str) -> os.PathLike: ...
    def get_paths(self) -> dict[str, os.PathLike]: ...
    def set_template_paths(
        self,
        dir: os.PathLike = ...,
        driver: os.PathLike = ...,
        comp: os.PathLike = ...,
    ) -> None: ...
    def set_gen_paths(
        self,
        dir: os.PathLike = ...,
        driver: os.PathLike = ...,
        comp: os.PathLike = ...,
    ) -> None: ...
    def set_driver_path(self, driver_file: str) -> None: ...
    def set_comp_path(self, comp_file: str) -> None: ...
    def set_extra_file_paths(
        self, templates: dict[str, str], gens: dict[str, str] | None = ...
    ) -> None: ...
    def read_template_str(self, file: str) -> str: ...
