class HardwareDescriptionLanguage:
    def __init__(
        self,
        config_data: dict[
            str, str | dict[str, str | list[dict[str, str | int]]]
        ],
    ) -> None: ...
    def generate_boilerplate(self): ...
    def set_config_data(
        self,
        config_data: dict[
            str, str | dict[str, str | list[dict[str, str | int]]]
        ],
    ) -> None: ...
    def dump_summary(self) -> None: ...
    def generate_config_file(self) -> None: ...
    def __set_hdl_str(self, hdl_str: str) -> None: ...

def run() -> None: ...
